@@ -1,6 +1,5 @@
 # -*- coding:utf-8; tab-width:4; mode:python -*-
 
-import inspect
 import itertools
 
 import hamcrest
@@ -11,11 +10,6 @@ try:
 except ImportError:
     from py27_backports import total_ordering
 
-try:
-    from inspect import getcallargs
-except ImportError:
-    from py27_backports import getcallargs
-
 
 import safeunicode
 from .exc import *
@@ -37,7 +31,7 @@ def add_indent(text, indent=0):
     return "%s%s" % (' ' * indent, text)
 
 
-class InvocationList(list):
+class OperationList(list):
     def lookup(self, invocation):
         if not invocation in self:
             raise LookupError
@@ -56,120 +50,6 @@ class InvocationList(list):
         return str.join('\n', lines)
 
 
-def create_proxy(collaborator):
-    if collaborator is None:
-        return DummyProxy()
-
-    return Proxy(collaborator)
-
-
-class DummyProxy(object):
-    def assert_has_method(self, name):
-        pass
-
-    def assert_signature_matches(self, invocation):
-        pass
-
-    def same_method(self, name1, name2):
-        return name1 == name2
-
-    def collaborator_classname(self):
-        return None
-
-
-def get_class(something):
-    if inspect.isclass(something):
-        return something
-    else:
-        return something.__class__
-
-
-class Proxy(object):
-    def __init__(self, collaborator):
-        self.collaborator = collaborator
-        self.collaborator_class = self.get_class()
-
-    def isclass(self):
-        return inspect.isclass(self.collaborator)
-
-    def get_class(self):
-        if self.isclass():
-            return self.collaborator
-        else:
-            return self.collaborator.__class__
-
-    def collaborator_classname(self):
-        return self.collaborator_class.__name__
-
-    def assert_signature_matches(self, invocation):
-        self.assert_has_method(invocation.name)
-        signature = Signature(self, invocation.name)
-        try:
-            signature.assert_match(invocation.context.args,
-                                   invocation.context.kargs)
-        except TypeError, e:
-            raise TypeError("%s.%s" % (self.get_class(), e))
-
-    def assert_has_method(self, name):
-        if not hasattr(self.collaborator, name):
-            reason = "'%s' object has no attribute '%s'" % \
-                (self.collaborator_classname(), name)
-            raise AttributeError(reason)
-
-    def same_method(self, name1, name2):
-        return getattr(self.collaborator, name1) == \
-            getattr(self.collaborator, name2)
-
-    def perform_invocation(self, invocation):
-        method = getattr(self.collaborator, invocation.name)
-        return method(*invocation.context.args,
-                       **invocation.context.kargs)
-
-
-class Signature(object):
-    """colaborator method signature"""
-    def __init__(self, proxy, name):
-        self.proxy = proxy
-        self.name = name
-        self.method = getattr(proxy.collaborator, name)
-        self.argspec = inspect.getargspec(self.method)
-
-#        print "signature:", self.method
-#        print "class:    ", self.method.__class__
-#        print "args(%s):   %s" % (len(self.argspec.args), self.argspec.args)
-#        print "varargs:  ", self.argspec.varargs
-#        print "keywords: ", self.argspec.keywords
-#        print "defaults: ", self.argspec.defaults
-
-#    def count_positionals(self):
-#        if self.argspec.defaults is None:
-#            ndefaults = 0
-#        else:
-#            ndefaults = len(self.argspec.defaults)
-#
-#        return len(self.argspec.args) - ndefaults - 1
-#
-#    def get_keywords(self):
-#        if self.argspec.defaults is None:
-#            return []
-#
-#        return self.argspec.args[-len(self.argspec.defaults):]
-
-    def assert_match(self, args, kargs):
-        if ANY_ARG in args:
-            return
-
-        if self.proxy.isclass():
-            args = (None,) + args  # self
-
-        getcallargs(self.method, *args, **kargs)
-
-    def __repr__(self):
-        return "%s.%s%s" % (self._proxy.collaborator_classname(),
-                            self.name,
-                            inspect.formatargspec(*self.argspec))
-
-
 class Observable(object):
     def __init__(self):
         self.observers = []
@@ -196,8 +76,8 @@ class Method(Observable):
         return self.double._manage_invocation(invocation)
 
     def create_invocation(self, args, kargs):
-        return Invocation(self.double, self.name,
-                          InvocationContext(*args, **kargs))
+        return Invocation.from_args(
+            self.double, self.name, args, kargs)
 
     def _was_called(self, context, times):
         invocation = Invocation(self.double, self.name, context)
@@ -247,10 +127,14 @@ def func_raising(e):
 
 @total_ordering
 class Invocation(object):
-    def __init__(self, double, name, context):
+    def __init__(self, double, name, context=None):
         self.double = double
         self.name = name
-        self.context = context
+        self.context = context or InvocationContext()
+
+    @classmethod
+    def from_args(cls, double, name, args=(), kargs={}):
+        return Invocation(double, name, InvocationContext(*args, **kargs))
 
     def delegates(self, delegate):
         if callable(delegate):
@@ -305,7 +189,8 @@ class Invocation(object):
         return False
 
     def __repr__(self):
-        return "%s.%s%s" % (self.double._classname(), self.name, self.context)
+        return "%s.%s%s" % (self.double._classname(),
+                            self.name, self.context)
 
     def show(self, indent=0):
         return add_indent(self, indent)
@@ -398,5 +283,62 @@ class InvocationFormatter(object):
                 for key, val in sorted(kargs.items())]
 
 
+class PropertyGet(Invocation):
+    def __repr__(self):
+        return "get %s.%s" % (self.double._classname(), self.name)
+
+
+class PropertySet(Invocation):
+    def __init__(self, double, name, value=None):
+        super(PropertySet, self).__init__(
+            double, name, InvocationContext(value))
+        self.value = value
+
+    def _was_called(self, times):
+        return self.double._was_called(self, times)
+
+    def __repr__(self):
+        return "set %s.%s to %s" % (
+            self.double._classname(), self.name, self.value)
+
+
+class Property(object):
+    def __init__(self, double, key):
+        self.double = double
+        self.key = key
+        self.value = None
+
+    def __get__(self, obj, type=None):
+        self._manage(PropertyGet(self.double, self.key))
+        return self.value
+
+    def __set__(self, obj, value):
+        prop = self.double._proxy.get_class_attr(self.key)
+        if prop.fset is None:
+            raise AttributeError("can't set attribute")
+
+        self._manage(PropertySet(self.double, self.key, value))
+        self.value = value
+
+    def _manage(self, operation):
+        self.double._manage_invocation(operation, check=False)
+
+
+class AttributeFactory(object):
+    typemap = {
+        'instancemethod': Method,
+        'property':       Property
+        }
+
+    @classmethod
+    def create(cls, double, key):
+        typeid = double._proxy.get_attr_typeid(key)
+        return cls.typemap[typeid](double, key)
+
+
+class SpyBase(object):
+    pass
+
+
 class MockBase(object):
     pass
